/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Mike Squillace - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.accservice.core;

import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.traverse.AbstractTreeNodeWalker;


/**
 * implementation of <code>TreeWalker</code> for walking a tree of <code>IAccessibleElement</code objects.
 *
 * @see org.eclipse.actf.accservice.core.IAccessibleElement
 * @author Mike Squillace
 *
 */
public class AccessibleElementTreeWalker extends AbstractTreeNodeWalker
{

	public AccessibleElementTreeWalker () {
		super();
	}

	public Object[] getChildren (Object element)
		throws InvalidComponentException {
		Object[] children = new Object[0];
		Object bridgedChild = getBridgedChild(element);
		if (bridgedChild != null) {
			children = new Object[] {bridgedChild};
		}else if (element instanceof IAccessibleElement) {
			IAccessibleElement acc = (IAccessibleElement) element;
			children = acc.getAccessibleChildren();
		}
		return children;
	}
	
	/** {@inheritDoc} */
	public Object getChild (Object element, int index) throws InvalidComponentException {
		return element instanceof IAccessibleElement ? ((IAccessibleElement) element).getAccessibleChild(index) : null;
	}

	/** {@inheritDoc} */
	public Object getParent (Object element) throws InvalidComponentException {
		Object parent = null;
		Object bridgedParent = getBridgedParent(element);
		if (bridgedParent != null) {
			parent = bridgedParent;
		}else if (element instanceof IAccessibleElement) {
			parent = ((IAccessibleElement) element).getAccessibleParent();
		}
		return parent;
	}

	/** {@inheritDoc} */
	public boolean hasChildren (Object element)
		throws InvalidComponentException {
		return element instanceof IAccessibleElement
				&& ((IAccessibleElement) element).getAccessibleChildCount() > 0;
	}

	public Object[] getStartNodes () {
		return new Object[0];
	}
}
