/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.core.win32.msaa;

import java.awt.Point;

import org.eclipse.actf.accservice.AccessibilityServiceException;
import org.eclipse.actf.accservice.IAccessibilityEventService;
import org.eclipse.actf.accservice.IAccessibilityService;
import org.eclipse.actf.accservice.IWindowService;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.model.InvalidComponentException;


public class MsaaAccessibilityService implements IAccessibilityService {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5517183072097924795L;

	/**
	 * name of dynamic module for MSAA access; value is 'actf-msaa'
	 */
	public static final String ACTF_MSAA_LIBRARY_NAME = "actf-msaa";

	/**
	 * accessibility service name for this service; value is 'msaa'
	 */
	public static final String MSAA_ACCSERVICE_NAME = "msaa";

	private IWindowService _windowService = new MsaaWindowService();
	private IAccessibilityEventService _accEventService = new MsaaAccessibilityEventService(_windowService);
	
	public IAccessibleElement createAccessibleElement(Object obj, Object[] params)
			throws InvalidComponentException {
		MsaaAccessibilityService.internalCoInitialize();
		MsaaAccessible mAcc = null;

		if (obj instanceof Point) {
			mAcc = new MsaaAccessible((Point) obj);
		} else if (obj instanceof Integer) {
			int hwnd = ((Integer) obj).intValue();
			int childId = -1;
			if (params != null) {
				int length = params.length;

				if (length == 1 && (params[0] instanceof Integer)) {
					childId = ((Integer) params[0]).intValue();
				}
			}
			mAcc = new MsaaAccessible(hwnd, childId);
		} else {
			mAcc = new MsaaAccessible(obj);
		}

		return mAcc;
	}

	public IAccessibilityEventService getAccessibilityEventService() {
		return _accEventService;
	}

	public IWindowService getWindowService() {
		return _windowService;
	}

	public void initialize() throws AccessibilityServiceException {
		try {
			System.loadLibrary(ACTF_MSAA_LIBRARY_NAME);
		} catch (Throwable e) {
				throw new AccessibilityServiceException("Unable to load library " + ACTF_MSAA_LIBRARY_NAME, e);
		}
	}
	
	protected static native boolean internalCoInitialize();
	protected static native boolean internalCoUnInitialize();


}
