/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.locate;

import java.util.Properties;

import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.locate.AbstractNodeLocator;


/**
 * general purpose locator for all instances of <code>IAccessibleElement</code>
 * 
 * @author Mike Squillace
 *
 */
public class AccessibleNodeLocator extends AbstractNodeLocator
{

	public AccessibleNodeLocator () {
		super("accessibleName", "msaa", new AccessibleNodePointerFactory());
	}

	/**
	 * identifies the specified <code>IAccessibleElement</code> with its accessibleName properties. 
	 * Locator properties are not read from any model-specific configuration file.
	 * 
	 * @param element element to be identified
	 * @return identification string for element, usually its accessibleName
	 */
	public Properties describe (Object element) {
		String name = null;
		Properties properties = new Properties();
		
		if (element instanceof IAccessibleElement) {
			IAccessibleElement acc = (IAccessibleElement) element;
			try {
				name = acc.getAccessibleName();
			}catch (InvalidComponentException e) {
			}
		}
		
		if (name == null || name.length() == 0) {
			name = "<None>";
		}
		properties.setProperty("accessibleName", name);
		return properties;
		
	}

}