/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model;

import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.actf.model.event.IModelEventListener;
import org.eclipse.actf.model.event.ModelEventType;


/**
 * base implementation for <code>IGuiModel</code>. Clients should extend this class 
 * rather than attempting to implement <code>IGuiModel</code>.
 * 
 * @author Mike Squillace
 *
 */
public abstract class AbstractGuiModel extends AbstractModel
	implements IGuiModel
{

	/** keyed by event ids with values that are instances of <code>ModelEventType</code> */
	protected Map eventIdMap = new HashMap();

	/**
	 * create an IGuiModel implementation
	 * 
	 * @param modelType - name of model
	 */
	public AbstractGuiModel (String modelType) {
		super(modelType);
		initEventIdMap();
	}
	
	/** {@inheritDoc} */
	public ModelEventType getModelEventType (Object eventId) {
		return (ModelEventType) eventIdMap.get(eventId);
	}
	
	/** {@inheritDoc} */
	public boolean isVisible (Object comp) {
		return false;
	}

	/** {@inheritDoc}
	 *  
	 *  default implementation merely checks that the component is not <code>null</code>.
	 */
	public boolean isValid (Object comp) {
		return comp != null;
	}

	/** {@inheritDoc} */
	public boolean requestFocusFor (Object comp) {
		return false;
	}

	/** {@inheritDoc} */
	public void highlight (Object element) {
	}

	/** {@inheritDoc}
	 *  
	 *  default implementation returns <code>null</code>
	 */
	public Rectangle getBoundingRectangle (Object element) {
		return null;
	}

	/** {@inheritDoc}
	 *  
	 *  default implementation returns <code>null</code>
	 */
	public ModelEventType[] getModelEventTypes (Class c) {
		return null;
	}
	
	/** {@inheritDoc}
	 * 
	 * equivalent to calling <code>registerModelEventListener(listener, eventTypes, null)</code>
	 */
	public void registerModelEventListener (IModelEventListener listener, ModelEventType[] eventTypes) {
		registerModelEventListener(listener, eventTypes, null);
	}
	
	/** {@inheritDoc} */
	public void registerModelEventListener (IModelEventListener listener, ModelEventType[] eventTypes, Object[] params) {
	}
	
	public void unregisterModelEventListener (IModelEventListener listener, ModelEventType[] eventTypes)  {
	}
	
	/**
	 * used to initialize the eventId map, which is keyed by eventId that corresponds to a <code>ModelEventType</code> 
	 * instance. Event ids are model-specific and typically defined as part of that model's definition.
	 * 
	 * @see org.eclipse.actf.model.event.ModelEventType
	 * @see #registerModelEventListener(IModelEventListener, ModelEventType[])
	 */
	protected void initEventIdMap () {
	}
	
} // AbstractGuiModel
