/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Mike Squillace - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.model.javapp.traverse;

import java.awt.Component;
import java.awt.Container;

import org.eclipse.actf.model.DefaultModelFactory;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.javapp.SwingGuiModel;
import org.eclipse.actf.model.traverse.AbstractTreeNodeWalker;

/**
 * implementation of a TreeWalker for the Java Swing GUI framework. Nodes 
 *  are typically instances of <code>java.awt.Container</code>.
 *  
 * @author Mike Squillace
 */
public class ContainerTreeNodeWalker extends AbstractTreeNodeWalker
{

	private SwingGuiModel _model;

	public ContainerTreeNodeWalker () {
		super();
		_model = (SwingGuiModel) DefaultModelFactory.getInstance().resolveModel(
			SwingGuiModel.SWING_MODEL);
	}

	/** {@inheritDoc} */
	public Object[] getChildren (Object element)
		throws InvalidComponentException {
		Object[] children = new Object[0];
		Object bridgedChild = getBridgedChild(element);
		if (bridgedChild != null) {
			children = new Object[] {bridgedChild};
		}else if (element instanceof Container) {
			children = ((Container) element).getComponents();
		}
		return children;
	}

	/** {@inheritDoc} */
	public Object getParent (Object element) throws InvalidComponentException {
		Object parent = null;
		Object bridgedParent = getBridgedParent(element);
		if (bridgedParent != null) {
			parent = bridgedParent;
		}else if (element instanceof Component) {
			parent = ((Component) element).getParent();
		}
		return parent;
	}

	public Object[] getStartNodes () {
		Object[] startNodes = new Object[0];
		try {
			Class eventMonitorType = Class.forName("com.sun.java.accessibility.util.EventQueueMonitor");
			startNodes = (Object[]) eventMonitorType.getMethod(
				"getTopLevelWindows", (Class[]) null).invoke(null, (Object[]) null);
		}catch (Exception e) {
		}
		return startNodes;
	}
} // ContainerTreeNodeWalker
