/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 
package org.eclipse.actf.validation;

import java.io.InputStream;

import org.eclipse.actf.core.ActfCorePlugin;
import org.eclipse.actf.core.config.AbstractConfiguration;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.ModelPlugin;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.EclipseResourceLocator;
import org.eclipse.actf.validation.core.ValidationProcessManager;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.rulebase.RuleBaseMapDocument;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The ValidationCorePlugin class controls the plug-in life cycle
 */
public class ValidationCorePlugin extends ModelPlugin {
	
		// The plug-in ID

	public static final String VALIDATION_CORE_PLUGIN_ID = "org.eclipse.actf.validation";

	// The shared instance
	private static ValidationCorePlugin plugin;

	/**
	 * The constructor
	 */
	public ValidationCorePlugin() {
		super();
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.Plugins#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);

		if (configuration
				.getSymbolPoolContents(ValidationConstants.VALIDATION_ID) == null) {
			if (configuration instanceof AbstractConfiguration) {
				((AbstractConfiguration) configuration)
						.createSymbolPool(ValidationConstants.VALIDATION_ID);
			}
		}
		configuration.setSymbolPool(ValidationConstants.VALIDATION_ID);
		configuration.setParameter(
				ValidationConstants.PERFORMING_VALIDATION_KEY, true);
		configuration.setParameter(ValidationConstants.VALIDATE_HIDDEN_KEY,
				false);
		configuration.setParameter(
				ValidationConstants.REVALIDATING_COMPONENTS_KEY, true);
		configuration.setParameter(ValidationConstants.REPORTER_LEVEL_KEY,
				IReporter.INFO);
		
		// set up script languages
		if (configuration.getSymbolPoolContents(ICodeProcessor.SCRIPTLANG_ID) == null) {
			if (configuration instanceof AbstractConfiguration) {
				((AbstractConfiguration) configuration)
						.createSymbolPool(ICodeProcessor.SCRIPTLANG_ID);

			}
		}
		configuration.setSymbolPool(ICodeProcessor.SCRIPTLANG_ID);
		configuration.setParameter("javascript", "org.eclipse.actf.validation.valdoc.scripting.JavaScriptCodeInterpreter");
		configuration.setSymbolPool(IConfiguration.ACTF_ID);
		configuration.setParameter(ICodeProcessor.FACTORY_PROCESSOR_KEY, ICodeProcessor.DEFAULT_CODEPROCESSOR_FACTORY );

		// load rulebases.xml files
		 RuleBaseManager rbMgr = RuleBaseManager.getInstance();
		 EclipseResourceLocator locator = (EclipseResourceLocator) runtimeContext.getResourceLocator();
		 locator.registerBundleName(ModelPlugin.MODEL_PLUGIN_ID);
		InputStream srcStream = locator.getResourceAsStream(ValidationConstants.DEFAULT_RULEBASE_MAP_FILE, null, "xml", getPluginId());
		if (srcStream != null) {
			rbMgr.buildRuleBases(new RuleBaseMapDocument(srcStream));
			srcStream.close();
		} else {
			LoggingUtil.println(IReporter.WARNING, "No rulebases found for plug-in " + getPluginId());
		}
		 
		//initialize the ValidationProcessManager
		ValidationProcessManager vpm = ValidationProcessManager.getInstance();
		
		// insure all accessibility services are registered
		// removed dependency on accservice by using reflection but this
		// is certainly not desirable and should not be necessary once extension points are used
		Bundle accService = Platform.getBundle("org.eclipse.actf.accservice");
		accService.start();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static ActfCorePlugin getDefault() {
		return plugin;
	}

	public String getPluginId() {
		return VALIDATION_CORE_PLUGIN_ID;
	}

}
