/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.Map;

/**
 * used to associated validation processes and validators.
 * 
 * @author John Buslawski
 * 
 */
public interface IValidationProcessMappingFactory {

	/**
	 * registers a validation process for the given validator
	 * 
	 * @param validatorName -
	 *            name of the validator under which the validation process is
	 *            being registered
	 * @param validationProcessName -
	 *            the class name of the validation process for this validator
	 */
	public void registerValidationProcessMapping(String validatorName,
			String validationProcessName);

	/**
	 * gets the validation process class name for this validator name.
	 * 
	 * @param name -
	 *            name of the validator class
	 * @return the name of the validation process class that uses this validator
	 */
	public String getValidationProcessMapping(String name);

	/**
	 * gets all of the mappings for validators and validation processes
	 * 
	 * @return the map of validator name/validation process names
	 */
	public Map getValidationProcessMappings();
}
