/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.lang.reflect.Member;

/**
 * an execution point that serves as a trigger for a validation from a method 
 * or constructor invocation.
 * 
 * @author Mike Squillace
 *
 */
public class MethodExecutionPoint extends ExecutionPoint
{

	protected Object[] arguments;

	protected Member member;

	public MethodExecutionPoint (String name, String validatorClassName,
									Object target, Object[] arguments,
									Member member) {
		super(name, validatorClassName, target);
		this.arguments = arguments;
		this.member = member;
	}

	/**
	 * @return the arguments to the member
	 */
	public Object[] getArguments () {
		return arguments;
	}

	/**
	 * @return the member (either constructor or method)
	 */
	public Member getMember () {
		return member;
	}

	/**
	 * return an ExecutionPoint of the same type but with a different name and
	 * validator class name. This is typically used in the Validation loop where
	 * we have a child class of a compoment that has a different model than
	 * the parent.
	 * @param name
	 * @param validator
	 * @param execPoint
	 * @return ExecutionPoint
	 */
	public ExecutionPoint cloneExecutionPoint (String name, String validator,
												ExecutionPoint execPoint) {
		MethodExecutionPoint newExecPoint = null;
		if (execPoint.getClass().isAssignableFrom(this.getClass())) {
			newExecPoint = new MethodExecutionPoint(name, validator, execPoint.getTarget(), ((MethodExecutionPoint) execPoint).getArguments(), ((MethodExecutionPoint) execPoint).getMember());
		}else {
			throw new IllegalArgumentException("ExecutionPoint type is not an MethodExecutionPoint.");
		}
		return newExecPoint;
	}
}
