/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.validators.IValidationContext;


/**
 * ValidationProcessState is a holder for the current state of a
 * IValidationProcess. This class is commmonly used during cross-model
 * validations, for instance, when an SWT GUI contains Swing components. During
 * any validation, the current ExecutionPoint, IRuleBase, and IValidationContext
 * are key elements needed to validate one component to the next. This hold
 * allow the current state of each of these elements to be saved and used again,
 * if necessary.
 * 
 * @author John Buslawski
 */
public class ValidationProcessState {

	private ExecutionPoint execPoint;

	private IRuleBase ruleBase;

	private IValidationContext context;

	/**
	 * @param execPoint
	 * @param iRuleBase
	 * @param context
	 */
	public ValidationProcessState(ExecutionPoint execPoint, IRuleBase iRuleBase,
			IValidationContext context) {
		this.execPoint = execPoint;
		ruleBase = iRuleBase;
		this.context = context;
	}

	/**
	 * @return the context
	 */
	public IValidationContext getValidationContext() {
		return context;
	}

	/**
	 * @param context
	 *            the context to set
	 */
	public void setValidationContext(IValidationContext context) {
		this.context = context;
	}

	/**
	 * @return the execPoint
	 */
	public ExecutionPoint getExecutionPoint() {
		return execPoint;
	}

	/**
	 * @param execPoint
	 *            the execPoint to set
	 */
	public void setExecutionPoint(ExecutionPoint execPoint) {
		this.execPoint = execPoint;
	}

	/**
	 * @return the ruleBase
	 */
	public IRuleBase getRuleBase() {
		return ruleBase;
	}

	/**
	 * @param iRuleBase
	 *            the ruleBase to set
	 */
	public void setRuleBase(IRuleBase iRuleBase) {
		ruleBase = iRuleBase;
	}
}
