/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.events;

/**
 * register with a validation launcher to receive notifications of various
 * stages of a validation reporting process.
 * 
 * @author Mike Squillace
 * 
 */
public interface IReportListener {

	/**
	 * call-back indicating that a validation report has been started.
	 * Validation reports are started only after the creation contexts have been
	 * created.
	 * 
	 * @param e
	 */
	public void reportStarted(ReportEvent e);

	/**
	 * call-back indicating that a validation report has ended. Other creation
	 * contexts and reports may follow a validation report.
	 * 
	 * @param e
	 */
	public void reportCompleted(ReportEvent e);

	/**
	 * call-back indicating that a creation context for an instantiated element
	 * has begun. After a set of creation contexts has been processed, a
	 * validation report typically follows.
	 * 
	 * @param e
	 */
	public void creationContextStarted(ReportEvent e);

	/**
	 * call-back indicating that a creation context has ended. Creation contexts
	 * are typically followed by a validation report.
	 * 
	 * @param e
	 */
	public void creationContextCompleted(ReportEvent e);
}
