/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.mozdom.staticweb.ui.browser;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.actf.validation.ui.jobs.IValidatableContainer;
import org.eclipse.actf.validation.ui.jobs.ValidationJob;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

public class MozillaDomBrowserEditor extends EditorPart
	implements IValidatableContainer
{

	protected BrowserControl browserCtrl;

	private ValidationJob _job;

	private Object _validatable;

	private boolean _auto = true;

	public MozillaDomBrowserEditor () {
		super();
	}

	public void doSave (IProgressMonitor monitor) {
	}

	public void doSaveAs () {
	}

	public void init (IEditorSite site, IEditorInput input)
		throws PartInitException {
		IResource resource = null;
		URL url = null;
		if (input instanceof MozillaBrowserEditorInput) {
			try {
				url = new URL(((MozillaBrowserEditorInput) input).getUrl());
				setSite(site);
				setInput(input);
			} catch (MalformedURLException e) {
				throw new PartInitException("Error opening ACTF Mozilla validation browser: invalid url");
			}
		} else if (input instanceof IFileEditorInput) {
			try {
				IFileEditorInput fileInput = (IFileEditorInput) input;
				resource = fileInput.getFile();
				url = resource.getRawLocation().toFile().toURL();
				setSite(site);
				setInput(new MozillaBrowserEditorInput(fileInput));
			} catch (Exception e) {
				throw new PartInitException("Error opening ACTF Mozilla validation browser");
			}
		} else {
			throw new PartInitException("Unknown input type to ACTF Mozilla validation browser");
		}

		_job = new MozillaValidationJob(url);
		_job.setValidatableContainer(this);
		_job.setResource(resource);
}

	public boolean isDirty () {
		return false;
	}

	public boolean isSaveAsAllowed () {
		return false;
	}

	public void createPartControl (Composite parent) {
		Composite main = new Composite(parent, SWT.NONE);
		main.setLayout(new FillLayout());
		browserCtrl = new MozillaDomBrowserControl(main, this);
		browserCtrl.setLocation(((MozillaBrowserEditorInput) getEditorInput()).getUrl());
	}

	public void setFocus () {
		browserCtrl.getBrowser().setFocus();
	}

	public Object getValidatable () {
		return _validatable;
	}

	public void setValidatable (Object validatable) {
		_validatable = validatable;
	}

	public ValidationJob getValidationJob () {
		return _job;
	}

	public void setValidationJob (ValidationJob job) {
		_job = job;
	}

	public boolean isAutoValidating () {
		return _auto;
	}

	public void setAutoValidating (boolean auto) {
		_auto = auto;
	}
}
