/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  <a href="mailto:masquill@us.ibm.com>Mike Squillace</a> - initial API and implementation
*******************************************************************************/ 


package org.eclipse.actf.validation.rulebase;

/**
 * interface for describing a rulebase to be used during validations.
 * 
 * @author <a href="mailto:masquill@us.ibm.com>Mike Squillace</a>
 *
 */
public interface IRuleBaseDescriptor
{

	/**
	 * gets the name of the rulebase being described by this descriptor
	 * 
	 * @return name of rulebase
	 */
	public String getRuleBaseName ();
	
	/**
	 * sets the name of the rulebase being described. The name must not be <code>null</code>.
	 *  
	 * @param name name of rulebase being described
	 */
	public void setRuleBaseName (String name);
	
	/**
	 * gets the type of the rulebase being described
	 * 
	 * @return type of rulebase being described by this descriptor
	 */
	public String getType ();
	
	/**
	 * sets the type of the rulebase being described
	 * 
	 * @param type
	 */
	public void setType (String type);
	
	/**
	 * get the class name of the factory for instantiating the rulebase being described by this descriptor
	 * 
	 * @return factory for instantiating the rulebase
	 */
	public String getRuleBaseFactory ();
	
	/**
	 * set the class name of the factory for instantiating rulebases described by this descriptor
	 * 
	 * @param factoryName
	 */
	public void setRuleBaseFactory (String factoryName);
	
}
