/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
* 
* Contributors:
* 	Mike Squillace - Initial API
* 	Phani Kommareddy - Implementor 
********************************************************************************/ 
package org.eclipse.actf.validation.rules;

import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Base implementation for a ICriterion
 * @see ICriterion
 * @see TypeCriterion
 * @see PropertyCriterion
 * @see CompoundCriterion
 */

public abstract class AbstractCriterion implements ICriterion {

	protected List<ICriterion> parts = new LinkedList<ICriterion>();
	protected Map<String, String> attributes = new HashMap<String, String>();
	protected ICriterion parent;
	protected IRuleSet ruleSet;
	
	public void addPart(ICriterion criterion) {
		parts.add(criterion);
	}
	
	public ICriterion[] getParts() {
		return (ICriterion[])parts.toArray(new ICriterion[0]);
	}
	
	public void setParts(ICriterion[] criterion) {
		parts = Arrays.asList(criterion);
	}

	public String getAttribute(String name) {
		return (String)attributes.get(name);
	}
	
	public void setAttribute(String name, String value) {
		attributes.put(name, value);
	}

	public ICriterion getParent() {
		return parent;
	}

	public void setParent(ICriterion parent) {
		this.parent = parent;
	}

	public IRuleSet getRuleSet() {
		return ruleSet;
	}

	public void setRuleSet(IRuleSet ruleSet) {
		this.ruleSet = ruleSet;
	}

	public int hashCode() {
		return ruleSet.getName().hashCode(); 
	}

	public boolean equals(Object other) {
		boolean result = false;
		if (other instanceof ICriterion) {
			ICriterion c = (ICriterion) other;
			result = ((c.getParent() == null && parent == null) || c.getParent().equals(parent))
				&& ((c.getParts() == null && parts == null) || c.getParts().equals(parts));
		}
		return result;
	}
}