/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import java.util.List;
import java.util.Map;

/**
 * basic map for mapping criteria to the validation rules list
 * that are to be applied to them 
 */
public interface ICriterionMapping {

	/**
	 * Returns the complete criterion -> List<IRule> mapping. 
	 * ICriterions are the keys in the map while IRules List are 
	 * the corresponding values. 
	 * 
	 * @return criteria to rules Map 
	 * 
	 */
	public Map<ICriterion, List<IRule>> getAllRules();

	/**
	 * Sets the complete map of Rules. ICriterion are the keys into the Map
	 * while the values are the corresponding IRule List.
	 * 
	 * @param criteriaToRulesMap
	 *             criteria to rules Map
	 * 
	 */
	public void setAllRules(Map<ICriterion, List<IRule>> criteriaToRulesMap);

	/**
	 * Retrieve a list of Rules for a given criterion 
	 * 
	 * @param criterion 
	 * 					a key in criteriaRules Map
	 * @return values for the key, List of rules
	 */
	public ICriterion[] getRulesForCriterion(ICriterion criterion);
	
	/**
	 * Return all criteria's i.e. the keys in the criteria to rules map. 
	 * @return keys in the criteria to Rules map
	 */
	public ICriterion[] getCriteria();
	
	
}
