/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.valdoc.rulebase.IValidationDocument;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.w3c.dom.Element;

/**
 * embodies actf:neq rule
 * 
 * @author John Buslawski
 * 
 */
public class NeqRule extends AbstractRule {

	private Object value;

	private String val;

	/** {@inheritDoc} */
	public void initialize(Object content) throws Exception {
		if (content instanceof Element) {
			processActfAttributes((Element) content);
			if ((val = ((Element) content).getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.NEQ)) != null) {
				try {
					value = Double.valueOf(val);
				} catch (Exception e) {
					value = val;
				}
			}
		}
	}

	/** {@inheritDoc} */
	public boolean apply(IValidationContext context, Object target) throws Exception {
		// execute the presence test
		boolean result = super.apply(context, target);
		// if not enabled, presence test failed,
		// or ignoring null target, then return
		if (isEnabled() && result && !isIgnoringNullTarget(target)) {
			try {
				if (target.equals(val)) {
					String message = findMessage();
					if (message == null || message.length() == 0) {
						message = "Value was equal to the specified value: " + val;
					}
					
					report(context, message);
					result = false;
				}
			} catch (Exception e) {
				LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
						"Exception caught while validating rule " + getName()
								+ " for target object " + target.getClass().getName(), e);
			}
		}
		
		return result;
	}
	
}
