/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.valdoc.rulebase.IValidationDocument;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.w3c.dom.Element;

/**
 * the actf:range rule
 * 
 * @author John Buslawski
 * 
 */
public class RangeRule extends AbstractRule {

	double minVal = 0.0;

	private double maxVal = 0.0;

	/** {@inheritDoc} */
	public void initialize(Object content) throws Exception {
		String rangeVal = null;
		if (content instanceof Element) {
			processActfAttributes((Element) content);
			if ((rangeVal = ((Element) content).getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.RANGE)) != null) {
				try {
					int comma = rangeVal.indexOf(',');
					if (comma > 0) {
						minVal = Double.parseDouble(rangeVal
								.substring(0, comma).trim());
						maxVal = Double.parseDouble(rangeVal.substring(
								comma + 1).trim());
					} else {
						LoggingUtil.println(IReporter.USER,
								"RangeRule.formApplication() : Malformed range: "
										+ rangeVal);
					}
				} catch (Exception e) {
					LoggingUtil
							.println(IReporter.SYSTEM_NONFATAL,
									"Exception caught while creating rule "
											+ getName(), e);
				}
			}
		}
	}

	/** {@inheritDoc} */
	public boolean apply(IValidationContext context, Object target) throws Exception {
		// execute the presence test
		boolean result = super.apply(context, target);
		// if not enabled, presence test failed,
		// or ignoring null target, then return
		if (isEnabled() && result && !isIgnoringNullTarget(target)) {
			if (!(target instanceof Number)
					|| ((Number) target).doubleValue() < minVal
					|| ((Number) target).doubleValue() > maxVal) {
				String message = findMessage();
				if (message == null || message.length() == 0) {
					StringBuffer msgBuff = new StringBuffer();
					msgBuff.append("Value is not within specified range: ");
					msgBuff.append(minVal);
					msgBuff.append("...");
					msgBuff.append(maxVal);
					message = msgBuff.toString();
				}
				
				report(context, message);
				result = false;
			}
		}
		
		return result;
	}
}
