/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import org.eclipse.actf.validation.valdoc.codeproc.EvaluationException;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.ValidationContextConstants;
import org.w3c.dom.Element;


/**
 * 
 */
public class ScriptRule extends AbstractRule {

	String expression = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.validation.rules.ComponentRule#formApplication(java.lang.Object)
	 */
	public void initialize(Object content) throws Exception {
		if (content instanceof Element) {
			processActfAttributes((Element) content);
			expression = getTextTrim((Element) content);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.validation.rules.ComponentRule#apply(java.lang.Object)
	 */
	public boolean apply(IValidationContext context, Object target) throws Exception {
		// execute the presence test
		boolean result = super.apply(context, target);
		// if not enabled, presence test failed,
		// or ignoring null target, then return
		if (isEnabled() && result && !isIgnoringNullTarget(target) && expression.length() > 0) {
			ICodeProcessor codeProcessor = (ICodeProcessor) context.getParameter(ValidationContextConstants.CODE_PROCESSOR);
			Object res = Boolean.FALSE;
			try {
				codeProcessor.setLocal("propertyValue", target, "true");
				res = codeProcessor.evaluateArguments(expression, Boolean.class);
			} catch (EvaluationException e) {
				throw e;
			}
			if (res instanceof Boolean && !((Boolean) res).booleanValue()) {
				String message = findMessage();
				if (message == null || message.length() == 0) {
					message = "Failed script test: " + expression;
				}
				
				report(context, message);
				result = false;
			}
		}
		
		return result;
	}
}
