/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
* 
* Contributors:
* 	Mike Squillace - Initial API
* 	Phani Kommareddy - Implementor 
*******************************************************************************/

package org.eclipse.actf.validation.rules;

import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.valdoc.rulebase.IValidationDocument;
import org.eclipse.actf.validation.valdoc.rulebase.ValidationDocumentRuleBaseMetadata;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.w3c.dom.Element;

/**
 * Criterion for a generic component type. 
 */
public class TypeCriterion extends AbstractCriterion {

	private Class targetClass;
	private boolean polymorphic;
	private String type;
	
	public boolean apply(IValidationContext context, Object target) throws Exception {
		boolean returnValue = false;
		if(polymorphic) {
			if(targetClass != null && targetClass.isAssignableFrom(target.getClass())) {
				returnValue = true;
			}
		} else if(targetClass != null && targetClass.equals(target.getClass())) {
			returnValue = true;
		}	
		return returnValue;
	}

	public void initialize(Object content) throws Exception {
		if(content instanceof Element) {
			Element elem = (Element) content;
			String poly = elem.getAttributeNS(
					IValidationDocument.ACTF_NAMESPACE_URI,  RuleConstants.POLYMORPHIC);
			polymorphic = !poly.equals("false");
			boolean isActfNamespace = elem.getNamespaceURI() != null && 
				elem.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI);
			
			if(!isActfNamespace) {
				String className = getAliasValue(elem.getLocalName());
				if (className != null && className.length() > 0) {
					targetClass = ClassLoaderCache.getDefault().classForName(className);
				}
			}
		}
	}
	
	protected String getAliasValue(String aliasName) {
		return ((ValidationDocumentRuleBaseMetadata) getRuleSet().getRuleBase()
				.getMetadata()).getAliasValue(aliasName);
	}

	/**
	 * Returns the class against which target objects are teste
	 * @return Class
	 */
	public Class getTargetClass() {
		return targetClass;
	}
	
	/**
	 * Whether the object being tested must be an instance only of the target
	 * class or if it can be an instance of the target class or any of its 
	 * subclasses 
	 * @return boolean
	 */
	public boolean isPolymorphic() {
		return polymorphic;
	}
	
}