/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.dialogs;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.views.ReportView;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

public class ReportFilterDialog extends Dialog
{

	protected ReportView view = null;

	// severity level checkboxes
	protected Button configSeverityButton = null;

	protected Button detailSeverityButton = null;

	protected Button infoSeverityButton = null;

	protected Button errorSeverityButton = null;

	protected Button warningSeverityButton = null;

	protected Button severeSeverityButton = null;

	// severity level buttons
	protected Button selectAllButton = null;

	protected Button deselectAllButton = null;

	// restore Defaults button
	protected Button restoreDefaultsButton = null;

	// field values
	protected boolean configSeverityValue = false;

	protected boolean detailSeverityValue = false;

	protected boolean infoSeverityValue = true;

	protected boolean warningSeverityValue = true;

	protected boolean errorSeverityValue = true;

	protected boolean severeSeverityValue = false;

	public ReportFilterDialog (Shell parent, ReportView rv) {
		super(parent);
		view = rv;
	}

	protected void configureShell (Shell shell) {
		super.configureShell(shell);
		shell.setText("Validation Report View Filters");
		//shell.setSize(250,250);
	}

	protected Control createDialogArea (Composite parent) {
		initializeDialogUnits(parent);
		Composite area = (Composite) super.createDialogArea(parent);
		area.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridLayout layout = new GridLayout();
		area.setLayout(layout);
		createSeverityGroup(area);
		createDefaultsButton(area);
		loadSettings();
		return area;
	}

	protected void createDefaultsButton (Composite parent) {
		restoreDefaultsButton = new Button(parent, SWT.PUSH);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.RIGHT;
		restoreDefaultsButton.setLayoutData(gd);
		GuiUtils.configureControl(
			restoreDefaultsButton, "restoreDefaults",
			"Restore default selections", "Restore Defaults");
		SelectionListener listener = new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				setDefaults();
			}
		};
		restoreDefaultsButton.addSelectionListener(listener);
	}

	protected void createSeverityGroup (Composite parent) {
		// report items severities filter group
		Group severityGroup = new Group(parent, SWT.NULL);
		severityGroup.setText("Show items with severities of:");
		GridLayout layout = new GridLayout();
		GridData gd = new GridData(GridData.FILL_BOTH);
		// the width hint is provdided to prevent title bar truncation
		// and make things a little less crowded
		gd.widthHint = 250;
		severityGroup.setLayoutData(gd);
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = true;
		severityGroup.setLayout(layout);
		configSeverityButton = new Button(severityGroup, SWT.CHECK);
		configSeverityButton.setText("Configuration");
		configSeverityButton.setLayoutData(new GridData(GridData.FILL_BOTH));
		errorSeverityButton = new Button(severityGroup, SWT.CHECK);
		errorSeverityButton.setText("Error");
		errorSeverityButton.setLayoutData(new GridData(GridData.FILL_BOTH));
		detailSeverityButton = new Button(severityGroup, SWT.CHECK);
		detailSeverityButton.setText("Detail");
		detailSeverityButton.setLayoutData(new GridData(GridData.FILL_BOTH));
		warningSeverityButton = new Button(severityGroup, SWT.CHECK);
		warningSeverityButton.setText("Warning");
		warningSeverityButton.setLayoutData(new GridData(GridData.FILL_BOTH));
		infoSeverityButton = new Button(severityGroup, SWT.CHECK);
		infoSeverityButton.setText("Information");
		infoSeverityButton.setLayoutData(new GridData(GridData.FILL_BOTH));
		severeSeverityButton = new Button(severityGroup, SWT.CHECK);
		severeSeverityButton.setText("Severe");
		severeSeverityButton.setLayoutData(new GridData(GridData.FILL_BOTH));
		selectAllButton = new Button(severityGroup, SWT.PUSH);
		GuiUtils.configureControl(
			selectAllButton, "selectAll", "Select all severity levels",
			"Select All");
		SelectionListener listener = new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				selectAllSeverities(true);
			}
		};
		selectAllButton.addSelectionListener(listener);
		deselectAllButton = new Button(severityGroup, SWT.PUSH);
		deselectAllButton.setText("Deselect All");
		GuiUtils.configureControl(
			deselectAllButton, "deselectAll", "Deselect all severity levels",
			"Deselect All");
		listener = new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				selectAllSeverities(false);
			}
		};
		deselectAllButton.addSelectionListener(listener);
	}

	protected void selectAllSeverities (boolean b) {
		configSeverityValue = b;
		detailSeverityValue = b;
		infoSeverityValue = b;
		warningSeverityValue = b;
		errorSeverityValue = b;
		severeSeverityValue = b;
		setControls();
	}

	protected void setDefaults () {
		// severity level defaults are error, warning, severe
		configSeverityValue = false;
		detailSeverityValue = false;
		infoSeverityValue = true;
		warningSeverityValue = true;
		errorSeverityValue = true;
		severeSeverityValue = false;
		setControls();
	}

	protected void okPressed () {
		configSeverityValue = configSeverityButton.getSelection();
		detailSeverityValue = detailSeverityButton.getSelection();
		infoSeverityValue = infoSeverityButton.getSelection();
		warningSeverityValue = warningSeverityButton.getSelection();
		errorSeverityValue = errorSeverityButton.getSelection();
		severeSeverityValue = severeSeverityButton.getSelection();
		saveSettings();
		super.okPressed();
	}

	protected void saveSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			settings.put(ValidationUIConstants.KEY_SEVERITY_FILTERS_SAVED, true);
			settings.put(ValidationUIConstants.KEY_SEV_CONFIG, configSeverityValue);
			settings.put(ValidationUIConstants.KEY_SEV_DETAIL, detailSeverityValue);
			settings.put(ValidationUIConstants.KEY_SEV_INFO, infoSeverityValue);
			settings.put(ValidationUIConstants.KEY_SEV_WARNING, warningSeverityValue);
			settings.put(ValidationUIConstants.KEY_SEV_ERROR, errorSeverityValue);
			settings.put(ValidationUIConstants.KEY_SEV_SEVERE, severeSeverityValue);
		}catch (Exception e) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e.getMessage(), e);
		}
	}

	protected void loadSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			if (settings.getBoolean(ValidationUIConstants.KEY_SEVERITY_FILTERS_SAVED)) {
				configSeverityValue = settings.getBoolean(ValidationUIConstants.KEY_SEV_CONFIG);
				detailSeverityValue = settings.getBoolean(ValidationUIConstants.KEY_SEV_DETAIL);
				infoSeverityValue = settings.getBoolean(ValidationUIConstants.KEY_SEV_INFO);
				warningSeverityValue = settings.getBoolean(ValidationUIConstants.KEY_SEV_WARNING);
				errorSeverityValue = settings.getBoolean(ValidationUIConstants.KEY_SEV_ERROR);
				severeSeverityValue = settings.getBoolean(ValidationUIConstants.KEY_SEV_SEVERE);
			}else {
				setDefaults();
			}
		}catch (Exception e) { // if not found, use defaults
			setDefaults();
		}
		setControls();
	}

	protected void setControls () {
		configSeverityButton.setSelection(configSeverityValue);
		detailSeverityButton.setSelection(detailSeverityValue);
		infoSeverityButton.setSelection(infoSeverityValue);
		errorSeverityButton.setSelection(errorSeverityValue);
		warningSeverityButton.setSelection(warningSeverityValue);
		severeSeverityButton.setSelection(severeSeverityValue);
	}
}
