/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.filters;

import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;


public class ReportActiveFilter extends ViewerFilter
{

	/*
	 * Filters out ValidationReports based on their active property
	 * If the report is marked inactive then return false so the report will not be displayed
	 * 
	 * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public boolean select (Viewer viewer, Object parentElement, Object element) {
		boolean result = false;
		if (element instanceof ValidationReport) {
			ValidationReport report = (ValidationReport) element;
			if (!report.isPersistent()) { //transient reports are by nature active
				result = true;
			}else { //check the active property of the persistent report
				result = report.isActive();
				//TODO move function to reports feature
				/*String a = report.getProperty(ReportServices.ACTIVE_KEY);
				if (a == null || a.equalsIgnoreCase("F")) {
					result = false;
				}else {
					result = true;
				}*/
			}
		}
		return result;
	}
}
