/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.filters;

import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;


/*
 * Filters out ValidationReports based on their scope property
 * 
 * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
 */
public class ReportScopeFilter extends ViewerFilter
{

	private int currentScope = 0;

	private String scopeValue = null;

	// report scope constants
	public static final int SCOPE_ALL = 0;

	public static final int SCOPE_ADMIN = 1;

	public static final int SCOPE_NONADMIN = 2;

	public static final int SCOPE_GROUP = 3;

	public static final int SCOPE_TYPE = 4;

	public static final int SCOPE_TOOL = 5;

	public static final int SCOPE_SOURCE = 6;

	public ReportScopeFilter () {
		getCurrentFilterSettings();
	}

	private void getCurrentFilterSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			currentScope = settings.getInt(ValidationUIConstants.KEY_REPORT_SCOPE);
			switch (currentScope) {
			case SCOPE_ALL:
				scopeValue = null;
				break;
			case SCOPE_TOOL:
				scopeValue = settings.get(ValidationUIConstants.KEY_TOOL_SCOPE_VALUE);
				break;
			case SCOPE_SOURCE:
				scopeValue = settings.get(ValidationUIConstants.KEY_SOURCE_SCOPE_VALUE);
				break;
			}
		}catch (Exception e) {
			setDefaults();
		}
	}

	private void setDefaults () {
		currentScope = SCOPE_ALL;
		scopeValue = null;
	}

	public boolean select (Viewer viewer, Object parentElement, Object element) {
		boolean result = false;
		if (element instanceof ValidationReport) {
			ValidationReport report = (ValidationReport) element;
			//String user = report.getProperty(ReportServices.USERID_KEY);
			switch (currentScope) {
			case SCOPE_ALL:
				result = true;
				break;
			case SCOPE_TOOL:
				String tool = report.getTool();
				//TODO move function to reports feature
				//String tool = report.getProperty(ReportServices.TOOL_KEY);
				if (scopeValue != null && tool.equals(scopeValue)) {
					result = true;
				}
				break;
			case SCOPE_SOURCE:
				String source = report.getSource();
				if (scopeValue != null && source.equals(scopeValue)) {
					result = true;
				}
				break;
			//this set not currently used	
			/*case SCOPE_ADMIN:
			 break;
			 case SCOPE_NONADMIN:
			 break;
			 case SCOPE_GROUP:
			 break;
			 case SCOPE_TYPE:
			 break;*/
			}
		}
		return result;
	}

	public boolean isFilterProperty (Object element, String property) {
		return property.equals("scope");
	}

	/**
	 * @param value for scopes other than ALL, the scope value 
	 * i.e. tool, source, etc.
	 */
	public void setScope (int scope, String value) {
		currentScope = scope;
		scopeValue = value;
	}
}
