/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  barryf - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.preferences;

import java.util.ArrayList;
import java.util.StringTokenizer;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.preference.ListEditor;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;



/**
 * @author barryf
 *
 */
public class PropertiesEditor extends ListEditor
{

	private class PropertyInputDialog extends Dialog
	{


		protected Text keyField, valField;

		protected String key, value;

		public PropertyInputDialog (Shell parentShell, String keyText,
									String valText, IInputValidator validator) {
			super(parentShell);
			key = keyText;
			value = valText;
		}



		public String getKey () {
			return key;
		}

		public String getValue () {
			return value;
		}

		protected void createButtonsForButtonBar (Composite parent) {
			super.createButtonsForButtonBar(parent);
		}

		protected void buttonPressed (int buttonId) {
			if (buttonId == IDialogConstants.OK_ID) {
				key = keyField.getText();
				value = valField.getText();
			}else {
				key = value = null;
			}
			super.buttonPressed(buttonId);
		}

		protected Control createDialogArea (Composite parent) {
			Composite c = (Composite) super.createDialogArea(parent);
			getShell().setText("Edit Property");
			c.setLayout(new GridLayout(2, false));
			Label l1 = new Label(c, SWT.RIGHT);
			l1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			l1.setText("Key:");
			keyField = new Text(c, SWT.LEFT | SWT.SINGLE | SWT.BORDER);
			GridData gd1 = new GridData(GridData.FILL_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_BEGINNING
					| GridData.GRAB_HORIZONTAL);
			gd1.widthHint = 300;
			keyField.setLayoutData(gd1);
			keyField.setText(key == null ? "" : key);
			Label l2 = new Label(c, SWT.RIGHT);
			l2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			l2.setText("Value:");
			valField = new Text(c, SWT.LEFT | SWT.SINGLE | SWT.BORDER);
			GridData gd2 = new GridData(GridData.FILL_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_BEGINNING
					| GridData.GRAB_HORIZONTAL);
			gd2.widthHint = 300;
			valField.setLayoutData(gd2);
			valField.setText(value == null ? "" : value);
			return c;
		}
	}

	private static String delimiter = ValidationPreferenceInitializer.DELIMITER;

	protected static String propDelimiter = ValidationPreferenceInitializer.PROPDELIMITER;


	protected List list;

	public List getListControl (Composite parent) {
		list = super.getListControl(parent);
		list.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				setEditEnable();
			}

			public void widgetDefaultSelected (SelectionEvent e) {
				widgetSelected(e);
			}
		});
		return list;
	}

	protected void doFillIntoGrid (Composite parent, int numColumns) {
		super.doFillIntoGrid(parent, numColumns);
		//TODO - this seems to be dead code - this property doesn't exist anywhere,
		// or anything like it.
		if (!Boolean.getBoolean("com.ibm.haac.raven.suppressPropertiesLayoutFix")) {
			// fix layout bug preventing vertical scrollbars from showing up
			GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			gd.horizontalSpan = numColumns - 1;
			gd.grabExcessHorizontalSpace = true;
			int listHeight = list.getItemHeight() * 12;
			Rectangle trim = list.computeTrim(0, 0, 0, listHeight);
			gd.heightHint = trim.height;
			list.setLayoutData(gd);
			//list.addControlListener(new ControlAdapter() {
			//    public void controlResized(ControlEvent e) {
			//        printTree((Control)e.widget, 0);
			//    }
			//});
		}
	}

	/* *** stuff to dump tree to see what scroll bars don't work ***   
	 protected void printTree(Control ctl, int level) {
	 System.out.println("" + level + ":" + ctl + "," + ctl.getBounds());
	 Object ld = ctl.getLayoutData();
	 if (ld != null) {
	 printLayoutData(ld);
	 }
	 Composite p = ctl.getParent();
	 if (p != null) {
	 printTree(p, level + 1);
	 }
	 }
	 protected void printLayoutData(Object ld) {
	 System.out.println(ld);
	 if (ld instanceof GridData) {
	 GridData gd = (GridData)ld;
	 System.out.println(gd.horizontalSpan + "x" + gd.verticalSpan + "," +
	 gd.grabExcessHorizontalSpace +"," + 
	 gd.grabExcessVerticalSpace + "," +
	 gd.horizontalAlignment + "," + 
	 gd.verticalAlignment);
	 }
	 }
	 */
	protected void setEditEnable () {
		int sidx = list.getSelectionIndex();
		editButton.setEnabled(sidx >= 0);
	}

	protected Composite buttonBox;

	protected Button editButton;

	public Composite getButtonBoxControl (Composite parent) {
		Composite xbuttonBox = super.getButtonBoxControl(parent);
		if (buttonBox == null) {
			buttonBox = xbuttonBox;
			editButton = new Button(buttonBox, SWT.PUSH);
			editButton.setText("Edit");
			editButton.setFont(parent.getFont());
			GridData data = new GridData(GridData.FILL_HORIZONTAL);
			data.heightHint = convertVerticalDLUsToPixels(
				editButton, IDialogConstants.BUTTON_HEIGHT);
			int widthHint = convertHorizontalDLUsToPixels(
				editButton, IDialogConstants.BUTTON_WIDTH);
			data.widthHint = Math.max(widthHint, editButton.computeSize(
				SWT.DEFAULT, SWT.DEFAULT, true).x);
			editButton.setLayoutData(data);
			editButton.addSelectionListener(new SelectionAdapter() {

				public void widgetSelected (SelectionEvent e) {
					int sidx = list.getSelectionIndex();
					if (sidx >= 0) {
						String[] xsel = list.getSelection();
						String sel = xsel[0];
						int posn = sel.indexOf(PropertiesEditor.propDelimiter);
						if (posn >= 0) {
							String keyText = sel.substring(0, posn).trim();
							String valText = sel.substring(
								posn + PropertiesEditor.propDelimiter.length()).trim();
							PropertyInputDialog id = new PropertyInputDialog(getShell(), keyText, valText, null);
							int rc = id.open();
							if (rc == Window.OK) {
								list.remove(sidx);
								list.add(id.getKey()
										+ PropertiesEditor.propDelimiter
										+ id.getValue(), sidx);
							}
						}
					}
				}

				public void widgetDefaultSelected (SelectionEvent e) {
					widgetSelected(e);
				}
			});
		}
		setEditEnable();
		return xbuttonBox;
	}

	public PropertiesEditor () {
		super();
	}

	public PropertiesEditor (String name, String labelText, Composite parent) {
		super(name, labelText, parent);
	}

	protected String getNewInputObject () {
		String res = null;
		PropertyInputDialog id = new PropertyInputDialog(getShell(), "", "", null);
		int rc = id.open();
		if (rc == Window.OK) {
			res = id.getKey() + propDelimiter + id.getValue();
		}
		return res;
	}

	protected String createList (String[] items) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < items.length; i++) {
			sb.append(items[i]);
			sb.append(delimiter);
		}
		return sb.toString();
	}

	protected String[] parseString (String stringList) {
		StringTokenizer st = new StringTokenizer(stringList, delimiter);
		ArrayList res = new ArrayList(st.countTokens());
		while (st.hasMoreTokens()) {
			String item = st.nextToken().trim();
			if (item.length() > 0) {
				res.add(item);
			}
		}
		return (String[]) res.toArray(new String[0]);
	}
}
