/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Mike Squillace
*	        	Implementation - Phani Kommareddy
*******************************************************************************/ 
package org.eclipse.actf.validation.ui.providers;

import java.util.ArrayList;

import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rules.CompoundCriterion;
import org.eclipse.actf.validation.rules.ICriterion;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.rules.NodeNameCriterion;
import org.eclipse.actf.validation.rules.PropertyCriterion;
import org.eclipse.actf.validation.rules.TypeCriterion;
import org.eclipse.actf.validation.ui.views.RulesMasterDetailBlock;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;


public class RuleBaseLabelProvider implements ILabelProvider
{
	public Image getImage (Object element) {
		return null;
	}

	public String getText (Object element) {
		String text = null;
		if(element instanceof IRuleBase) {
			text = ((IRuleBase)element).getName();
		} else if(element instanceof IRuleSet) {
			text = ((IRuleSet)element).getName();
		} else if(element instanceof IRule) {
			IRule rule = (IRule)element; 
			text = getRuleText(rule);
			if(rule.isEnabled()) {
				RulesMasterDetailBlock.getCheckBoxTreeViewer().setChecked(element, true);
			} else {
				RulesMasterDetailBlock.getCheckBoxTreeViewer().setChecked(element, false);
			}
		} else if(element instanceof ICriterion){
			if(element instanceof TypeCriterion) {
				text = "TypeCriterion"; 
			} else if(element instanceof PropertyCriterion) {
				text = "PropertyCriterion";
			} else if(element instanceof CompoundCriterion) {
				text = "CompoundCriterion";
			} else if (element instanceof NodeNameCriterion){
				text = "NodeNameCriterion";
			} 
		}
		if (text == null) {
			text=element.toString();
		}
		return text;
	}

	public String getRuleText(IRule rule) {
		String text = rule.getName();
		ArrayList<String> result = new ArrayList<String>();
		String[] dotTokens = text.split("\\.");
		for(int i=0;i<dotTokens.length;i++) {
			if(dotTokens[i] != null) {
				String[] tokens = dotTokens[i].split(":");
				for(int j=0;j<tokens.length;j++) {
					result.add(tokens[j]);					
				}
			}
		}
		for(int i=0;i<result.size();i++) {
			String temp = result.get(i).toString();
			if(temp.equalsIgnoreCase("ReflexiveRule")) {
				text = "ReflexiveRule";
				break;
			} else if(temp.equalsIgnoreCase("OrRule")) {
				text = "OrRule";
				break;
			} else if(temp.equalsIgnoreCase("ScriptRule")) { 
				text = "ScriptRule";
				break;
			} else {				
				continue;
			}
		}
		return text;
	}
	
	public void addListener (ILabelProviderListener listener) {
	}

	public void dispose () {
		//Nothing to dispose
	}

	public boolean isLabelProperty (Object element, String property) {
		return false;
	}

	public void removeListener (ILabelProviderListener listener) {
		//listeners.remove(listener);
	}
}
