/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  barryf - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.views;

import java.awt.Container;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.actf.core.ActfCorePlugin;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.reports.ValidationReporter;
import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.actf.validation.ui.ISeverityColorProvider;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.filters.SeverityFilter;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.providers.ISeverityProvider;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author barryf
 * 
 */
public abstract class AbstractViewPart extends ViewPart
implements ISeverityProvider, ISeverityColorProvider, IStatusLineAccess {

	protected static class XSelectionListener extends
			GuiUtils.BaseSelectionAdapter {
	}

	protected static final String[] saType = new String[0];

	protected static final Class[] caType = new Class[0];

	public static final Object[] oaType = new Object[0];

	protected ActfCorePlugin corePlugin = ActfCorePlugin.getDefault();
	
	protected ValidationUiPlugin plugin = ValidationUiPlugin.getDefault();

	public boolean isDebugging(String option) {
		return plugin.isDebugging();
	}

	public ImageRegistry getImageRegistry() {
		return plugin.getImageRegistry();
	}

	public void registerImage(String key, String imageName) {
		plugin.registerImage(key, imageName);
	}

	/** name format is "key:imageName" */
	public void registerImages(String[] imageNames) {
		plugin.registerImages(imageNames);
	}

	public void registerImage(String key, Image image) {
		plugin.registerImage(key, image);
	}

	public void registerImage(String key, ImageDescriptor image) {
		plugin.registerImage(key, image);
	}

	public Image getRegisteredImage(String key) {
		return plugin.getRegisteredImage(key);
	}

	protected Composite viewParent;

	public Composite getViewParent() {
		return viewParent;
	}

	protected static Map awtGuis = new HashMap();

	public static Map getAwtGuis() {
		return awtGuis;
	}

	// *** rename variable, make sure no subclass references ***
	protected Viewer viewer;

	public Viewer getAssociatedViewer() {
		return viewer;
	}

	protected void setAssociatedViewer(Viewer viewer) {
		this.viewer = viewer;
	}

	protected void updateView(String id, Object arg1) {
		updateView(id, null, arg1);
	}

	protected void updateView(String id, Object arg1, Object arg2) {
		if (arg1 != null || arg2 != null) {
			IViewPart xview = findView(id);
			if (xview != null) {
				ViewPart cv = (ViewPart) xview;
				try {
					Class cvclass = cv.getClass();
					Method m = cvclass.getMethod("setInput",
							new Class[] { Object.class });
					if (m != null) {
						m.invoke(cv,
								new Object[] { arg1 != null ? arg1 : arg2 });
					}
				} catch (Exception e) {
					// e.printStackTrace();
					// *** temp removal ***
					// logException("Cannot update view", e);
				}
			}
		}
	}

	public void createPartControl(Composite parent) {
		viewParent = parent;
	}

	public void dispose() {
		super.dispose();
	}

	protected static Control findControl(Widget root, int id) {
		Control result = null;
		Method ctrlMeth = null, itemsMeth = null;
		try {
			if (root instanceof Item) {
				ctrlMeth = root.getClass().getMethod("getControl", (Class[]) null);
			} else {
				itemsMeth = root.getClass().getMethod("getItems", (Class[]) null);
			}
		} catch (Exception e) {
		}
		if (System.identityHashCode(root) == id) {
			if (root instanceof Control) {
				result = (Control) root;
			} else if (ctrlMeth != null) {
				try {
					result = (Control) ctrlMeth.invoke(root, (Object[]) null);
				} catch (Exception e) {
				}
			}
		} else if (itemsMeth != null) {
			try {
				Item[] children = (Item[]) itemsMeth.invoke(root, (Object[]) null);
				for (int i = 0; result == null && i < children.length; i++) {
					result = findControl(children[i], id);
				}
			} catch (Exception e) {
			}
		} else if (ctrlMeth != null) {
			try {
				result = findControl((Control) ctrlMeth.invoke(root, (Object[]) null), id);
			} catch (Exception e) {
			}
		} else if (root instanceof Composite) {
			Control[] children = ((Composite) root).getChildren();
			for (int i = 0; result == null && i < children.length; i++) {
				result = findControl(children[i], id);
			}
		}
		return result;
	}

	public static IViewPart findView(String id) {
		IViewPart result = null;
		IWorkbench wb = PlatformUI.getWorkbench();
		if (wb != null) {
			IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
			if (ww != null) {
				IWorkbenchPage wp = ww.getActivePage();
				if (wp != null) {
					result = wp.findView(id);
				}
			}
		}
		return result;
	}

	public static IViewPart activateView(String id) {
		IViewPart result = null;
		IWorkbench wb = PlatformUI.getWorkbench();
		if (wb != null) {
			IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
			if (ww != null) {
				IWorkbenchPage wp = ww.getActivePage();
				if (wp != null) {
					try {
						result = wp.showView(id);
					} catch (PartInitException e) {
						ValidationUiPlugin.getDefault().logException(e);
						e.printStackTrace();
					}
				}
			}
		}
		return result;
	}

	/**
	 * Create a report.
	 * 
	 * @param report
	 *            the validation report object
	 * @param description
	 *            report description
	 * @param component (=
	 *            source) component report is on
	 * @param tool
	 *            tool used to create the report
	 * @param arch
	 *            architecure of components
	 */
	public void createReport(final ValidationReport report,
			final String description, final String component,
			final String tool, final String arch) {
		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

			public void run() {
				final ReportManagerView dbview = (ReportManagerView) findView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
				if (dbview != null) {
					dbview
							.addReport(report, description, component, tool,
									arch);
				}
			}
		});
	}

	/**
	 * Create a set of reports.
	 * 
	 * @param reports
	 *            report array
	 * @param description
	 *            report description
	 * @param source
	 *            component report is on
	 * @param toolId
	 *            tool used to create the report
	 * @param model
	 *            model of components
	 * @return identifiers of created reports or null if no reports created
	 */
	public java.util.List createReports(final ValidationReport[] reports,
			final String description, final String source, final String toolId,
			final String model) {
		// final DatabaseReportView dbview =
		// (DatabaseReportView)findView(ValidationUIConstants.DATABASE_VIEW_ID);
		final ReportManagerView dbview = (ReportManagerView) findView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
		java.util.List result = null;
		if (dbview != null) {
			result = (java.util.List) executeWithWaitCursor(dbview
					.getViewParent(), new AbstractXRunnable() {

				public void run() throws Exception {
					// setResult(dbview.createReports(reports, description,
					// source, toolId, architecture));
					setResult(dbview.addReports(reports, description, source,
							toolId, model));
				}
			});
		}
		return result;
	}

	protected void removeChildren(Composite parent) {
		Control[] children = parent.getChildren();
		for (int i = 0; i < children.length; i++) {
			Control child = children[i];
			if (!child.isDisposed()) {
				child.dispose();
			}
		}
	} // end--removeChildren

	protected void removeComposite(Composite parent) {
		if (!parent.isDisposed()) {
			parent.dispose();
		}
	} // end--removeComposite

	public void update(Object c) {
		if (c instanceof Control) {
			update((Composite) c);
		} else {
			update((Container) c);
		}
	}

	public void update(Composite c) {
		c.layout();
		c.redraw();
		c.setVisible(true);
	}

	public void update(Container c) {
		c.validate();
		c.repaint();
	}

	protected void removeChildren(Container parent) {
		parent.removeAll();
	}

	abstract protected String getMessageTitle();

	protected void reportOk(String msg, String extra) {
		// trace("reportOk: " + msg + "\n" + extra);
		MessageDialog.openInformation(getViewParent().getShell(),
				getMessageTitle(), msg + "\n" + extra);
	}

	protected void reportError(String msg, String extra) {
		// trace("reportError: " + msg + "\n" + extra);
		MessageDialog.openError(getViewParent().getShell(), getMessageTitle(),
				msg + "\n" + extra);
		// reportControl(msg, MessageDialog.ERROR, extra);
	}

	protected void reportMessage(String message) {
		showMessage(message);
	}

	protected void showMessage(String message) {
		MessageDialog.openInformation(getViewParent().getShell(),
				getMessageTitle(), message);
	}

	protected ValidationReport lastReport;

	public ValidationReport getLastReport() {
		return lastReport;
	}

	protected void setLastReport(ValidationReport report) {
		lastReport = report;
		refreshAssociatedViewer();
	}

	public static void setReportViewInput(final ValidationReport report) {
		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
			public void run() {
				ReportView rv = (ReportView) findView(ValidationUIConstants.REPORT_VIEW_ID);
				if (rv != null) {
					rv.setInput(report);
				}
			}
		});
	}

	public void clearLastReportedErrors() {
		setLastReport(null);
	}

	protected static final Class[] xsaType = new Class[] { String[].class };

	protected void refreshAssociatedViewer() {
		Viewer viewer = getAssociatedViewer();
		if (viewer != null && viewer instanceof StructuredViewer) {
			StructuredViewer sviewer = (StructuredViewer) viewer;
			IBaseLabelProvider provider = sviewer.getLabelProvider();
			try {
				Class clazz = provider.getClass();
				Method m = clazz.getMethod("setErrors", xsaType);
				ValidationReport report = getLastReport();
				m.invoke(provider, new Object[] { report });
			} catch (Exception e) {
				// e.printStackTrace(); // *** test, ignore this in production
				// ***
			}
			viewer.refresh();
		}
	}

	protected static final Class[] args = {};

	protected void sizeTableViewerTable(Table t, Composite parent) {
		// t.setSize(600, 400);
		Point p = t.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		Display d = parent.getDisplay();
		Rectangle r = d.getBounds();
		Point s = new Point(Math.min(r.width - 400, Math.max(p.x, 600)), Math
				.min(r.height - 200, Math.max(p.y, 200)));
		t.setSize(s);
		parent.layout();
	}

	protected String wordCase(String s) {
		return ViewUtils.wordCase(s);
	}

	protected String[] split(String key, char delim) {
		return ViewUtils.split(key, delim);
	}

	protected String removeObjectId(String s) {
		return ViewUtils.removeObjectId(s);
	}

	public String getObjectId(String s) {
		return ViewUtils.getObjectId(s);
	}

	public String[] getSeverityKeys() {
		return SeverityFilter.severityKeys;
	}

	protected static final int defSeverity = IReporter.WARNING;

	protected static final int configSeverity = IReporter.CONFIG;

	protected static final int infoSeverity = IReporter.INFO;

	protected static final int warningSeverity = IReporter.WARNING;

	protected static final int errorSeverity = IReporter.ERROR;

	protected String[] severityKeys;

	public int getDefaultSeverity() {
		return defSeverity;
	}

	public int getConfigSeverity() {
		return configSeverity;
	}

	public int getInfoSeverity() {
		return infoSeverity;
	}

	public int getWarningSeverity() {
		return warningSeverity;
	}

	public int getErrorSeverity() {
		return errorSeverity;
	}

	public void parse(InputStream stream, DefaultHandler dh) throws IOException {
		final SAXParserFactory pf = SAXParserFactory.newInstance();
		try {
			SAXParser parser = pf.newSAXParser();
			InputSource is = new InputSource(stream);
			parser.parse(is, dh);
		} catch (RuntimeException e) {
			IOException ex = new IOException(e.getMessage());
			ex.initCause(e);
			throw ex;
		} catch (Exception e) {
			IOException ex = new IOException(e.getMessage());
			ex.initCause(e);
			throw ex;
		}
	}

	protected ValidationUiPlugin getActfPlugin() {
		return plugin;
	}

	protected Object executeWithWaitCursor(Control ctl, final IXRunnable r) {
		Cursor waitCursor = ctl.getDisplay().getSystemCursor(SWT.CURSOR_WAIT);
		ctl.getShell().setCursor(waitCursor);
		try {
			r.run();
		} catch (Exception e) {
			r.setException(e);
		} finally {
			ctl.getShell().setCursor(null);
		}
		return r.getResult();
	}

	protected IPreferenceStore getPreferenceStore() {	
		if (plugin == null) {
			plugin = ValidationUiPlugin.getDefault();
		}
		IPreferenceStore store = plugin.getPreferenceStore();
		return store;
	}

	protected int mapTraceLevel(String level) {
		int traceLevel = 3;
		if (level != null) {
			if (level.equals(ValidationPreferenceInitializer.V_ALL)) {
				traceLevel = 1;
			} else if (level.equals(ValidationPreferenceInitializer.V_INFO)) {
				traceLevel = 2;
			} else if (level.equals(ValidationPreferenceInitializer.V_WARNING)) {
				traceLevel = 3;
			} else if (level.equals(ValidationPreferenceInitializer.V_ERROR)) {
				traceLevel = 4;
			} else if (level.equals(ValidationPreferenceInitializer.V_NONE)) {
				traceLevel = 5;
			}
		}
		return traceLevel;
	}

	protected String getProperty(IResource res, String name) {
		return getProperty(res, "org.eclipse.actf.validation.ui", name, null);
	}

	protected String getProperty(IResource res, String name, String def) {
		return getProperty(res, "org.eclipse.actf.validation.ui", name, def);
	}

	protected String getProperty(IResource res, String qual, String name,
			String def) {
		String prop = null;
		try {
			String xprop = res.getPersistentProperty(new QualifiedName(qual,
					name));
			if (xprop != null) {
				xprop = xprop.trim();
				if (xprop.length() > 0) {
					prop = xprop;
				}
			}
		} catch (CoreException e) {
		}
		return prop != null ? prop : def;
	}


	protected void printLine(Printer printer, GC gc, int x, int y, String text) {
		gc.drawString(text, x, y);
	}

	protected void dispose(Set disposables) {
		for (Iterator i = disposables.iterator(); i.hasNext();) {
			Object o = i.next();
			i.remove();
			try {
				Class clazz = o.getClass();
				Method m = clazz.getMethod("dispose", (Class[]) null);
				m.invoke(o, (Object[]) null);
			} catch (Exception e) {
			}
		}
	}

	public IStatusLineManager getStatusLineManager() {
		return ((IViewSite) getSite()).getActionBars().getStatusLineManager();
	}

	public void displayStatusMessage(String message) {
		IStatusLineManager slm = getStatusLineManager();
		slm.setMessage(message);
	}

	public void displayStatusMessage(String message, Image image) {
		IStatusLineManager slm = getStatusLineManager();
		slm.setMessage(image, message);
	}

	public void displayErrorMessage(String message) {
		IStatusLineManager slm = getStatusLineManager();
		slm.setErrorMessage(message);
	}

	public void displayErrorMessage(String message, Image image) {
		IStatusLineManager slm = getStatusLineManager();
		slm.setErrorMessage(image, message);
	}

	public void clearMessages() {
		displayErrorMessage(null);
		displayStatusMessage(null);
	}

	protected abstract class BaseSelectionAdapter extends SelectionAdapter {

	
		public void widgetDefaultSelected(SelectionEvent e) {
			widgetSelected(e);
		}
	}

	public void enableToolbarButton(String toolItemId, boolean enabled) {
		IToolBarManager tbMgr = getViewSite().getActionBars()
				.getToolBarManager();
		IContributionItem item = tbMgr.find(toolItemId);
		if (item instanceof ActionContributionItem) {
			ActionContributionItem aci = (ActionContributionItem) item;
			aci.getAction().setEnabled(enabled);
			item.update(IAction.ENABLED);
		}
	}

	protected void initContextMenu(Viewer viewer) {
		initContextMenu(viewer, getViewId());
	}

	protected void initContextMenu(Viewer viewer, String id) {
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		fillContextMenu(menuMgr);
		Control ctl = viewer.getControl();
		if (ctl != null) {
			Menu menu = menuMgr.createContextMenu(ctl);
			ctl.setMenu(menu);
		}
		getSite().registerContextMenu(id, menuMgr, viewer);
	}

	protected void fillContextMenu(IMenuManager menuMgr) {
		// menuMgr.add(new Action("Test Action") {});
		menuMgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		menuMgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	protected String getViewId() {
		return getClass().getName();
	}

	public static final String ACCESSIBLE_IMAGE_NAME = "accessible";

	public static final String COMPONENT_IMAGE_NAME = "component";

	public static final String CONTAINER_IMAGE_NAME = "container";

	public Image getImageForElement(Object element, int index) {
		return ViewUtils.getImageForElement(element, index);
	}

	protected ValidationReporter createReporter(int traceLevel) {
		ValidationReporter reporter = new ValidationReporter();
		reporter.setDefaultOutputLevel(traceLevel);
		reporter.setSourceID(IReporter.VALIDATION);
		reporter.open();
		return reporter;
	}

	public Color getErrorBackgroundColor(Control ctl) {
		return (ValidationUiPlugin.getDefault()).getErrorBackgroundColor(ctl);
	}

	public Color getWarningBackgroundColor(Control ctl) {
		return (ValidationUiPlugin.getDefault()).getWarningBackgroundColor(ctl);
	}

	public Color getInfoBackgroundColor(Control ctl) {
		return (ValidationUiPlugin.getDefault()).getInfoBackgroundColor(ctl);
	}

	public Color getNormalBackgroundColor(Control ctl) {
		return (ValidationUiPlugin.getDefault()).getNormalBackgroundColor(ctl);
	}

	public Color getNormalForegroundColor(Control ctl) {
		return (ValidationUiPlugin.getDefault()).getNormalForegroundColor(ctl);
	}

	public static int indexOfIgnoreCase(List l, String s) {
		return ViewUtils.indexOfIgnoreCase(l, s);
	}

	public static int indexOfIgnoreCase(String[] l, String s) {
		return ViewUtils.indexOfIgnoreCase(l, s);
	}

	public static boolean runningOnWindows() {
		String osname = System.getProperty("os.name");
		return osname.toLowerCase().indexOf("windows") >= 0;
	}

	protected String getLibpath() {
		String newLP = System.getProperty("java.library.path");
		String libpath = getPreferenceStore().getString(
				ValidationPreferenceInitializer.P_LIBPATH);
		if (libpath != null && libpath.length() > 0) {
			newLP = libpath + File.pathSeparator + newLP;
		}
		return newLP;
	}
}
