/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Ann Ford
*	
*******************************************************************************/
package org.eclipse.actf.validation.ui.views;

import java.net.URL;

import org.eclipse.actf.validation.rulebase.IDocumentRuleBase;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

public class RulebaseDetailsPage extends AbstractDetailsPage {

	private IDocumentRuleBase rulebase;

	private Text versionText;
	private Text locationText;
	private Text modelText;

	public void createContents(Composite parent) {
		TableWrapLayout layout = new TableWrapLayout();
		layout.topMargin = 5;
		layout.leftMargin = 5;
		layout.rightMargin = 2;
		layout.bottomMargin = 2;
		parent.setLayout(layout);

		toolkit = mform.getToolkit();
		Section section = toolkit.createSection(parent, Section.TITLE_BAR);
		section.marginWidth = 10;
		section.setText("Rulebase Details");
		TableWrapData td = new TableWrapData(TableWrapData.FILL,
				TableWrapData.TOP);
		td.grabHorizontal = true;
		section.setLayoutData(td);
		Composite client = toolkit.createComposite(section);
		GridLayout glayout = new GridLayout();
		glayout.marginWidth = glayout.marginHeight = 0;
		glayout.numColumns = 2;
		client.setLayout(glayout);
		versionText = createLabelText(client, "Version", "");
		versionText.setEditable(false);
		modelText = createLabelText(client, "Model", "");
		modelText.setEditable(false);
		locationText = createLabelText(client, "Location", "");
		locationText.setEditable(false);
		toolkit.paintBordersFor(section);
		section.setClient(client);
	}

	@Override
	public void selectionChanged(IFormPart part, ISelection selection) {
		IStructuredSelection ssel = (IStructuredSelection) selection;
		if (ssel.size() == 1) {
			if (ssel.getFirstElement() instanceof IDocumentRuleBase) {
				rulebase = (IDocumentRuleBase) ssel.getFirstElement();

				if (rulebase != null) {
					versionText.setText(rulebase.getVersion() == null ? ""
							: rulebase.getVersion());
					modelText.setText(rulebase.getModelType() == null ? ""
							: rulebase.getModelType());
					URL location = rulebase.getDocumentLocation();
					locationText.setText(location.getFile() == null ? ""
							: location.getFile());
				}
			}
		}
	}

	public void refresh() {
		// TODO Auto-generated method stub

	}

}
