/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Ann Ford
*	
*******************************************************************************/
package org.eclipse.actf.validation.ui.views;

import java.util.ArrayList;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.rulebase.DocumentRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rulebase.IRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.filters.RuleViewerFilter;
import org.eclipse.actf.validation.ui.providers.RuleBaseContentProvider;
import org.eclipse.actf.validation.ui.providers.RuleBaseLabelProvider;
import org.eclipse.actf.validation.ui.providers.RuleViewerDetailsPageProvider;
import org.eclipse.actf.validation.ui.wizards.LoadRulebaseWizard;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;

public class RulesMasterDetailBlock extends MasterDetailsBlock {
	public static final String VERTICAL_ICON_FILENAME = "icons/vertical";
	public static final String HORIZONTAL_ICON_FILENAME = "icons/horizontal";
	public static final String COLLAPSE_ALL_ICON_FILENAME = "icons/collapseall";
	public static final String EXPAND_ALL_ICON_FILENAME = "icons/expandall";

	private static CheckboxTreeViewer tv = null;
	private RulebaseDetailsPage rulebaseDetailsPage = null;
	private RuleDetailsPage ruleDetailsPage = null;
	private RulebaseView rbv = null;
	private RuleBaseManager rbMgr = null;
	

	public RulesMasterDetailBlock(RulebaseView rulebaseView) {
		super();
		rbv = rulebaseView;
	}

	@Override
	protected void createMasterPart(final IManagedForm managedForm,
			Composite parent) {
		FormToolkit toolkit = managedForm.getToolkit();
		Section section = toolkit.createSection(parent, Section.TITLE_BAR);

		section.setText("Rule Bases and Rules");
		section.marginWidth = 10;
		section.marginHeight = 5;
		Composite composite = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		composite.setLayout(layout);
		Tree tree = toolkit.createTree(composite, SWT.SINGLE | SWT.CHECK
				| SWT.BORDER);
		Composite buttonGroup = toolkit.createComposite(composite, SWT.WRAP);
		buttonGroup.setLayout(new FillLayout(SWT.VERTICAL));
		Button addButton = toolkit
				.createButton(buttonGroup, "&Add...", SWT.PUSH);
		addButton.setToolTipText("Add a new rulebase from a file or URL");
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				Shell shell = rbv.getSite().getShell();
				LoadRulebaseWizard wizard = new LoadRulebaseWizard();
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.create();
				if (dialog.open() == IDialogConstants.OK_ID) {
					loadViewer(false);
				}
			}

		});
		Button reloadButton = toolkit.createButton(buttonGroup, "&Reload",
				SWT.PUSH);
		reloadButton.setToolTipText("Reload the selected rulebase");
		reloadButton.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				loadViewer(true);
			}

		});
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 20;
		gd.widthHint = 100;
		tree.setLayoutData(gd);
		section.setClient(composite);
		final SectionPart spart = new SectionPart(section);
		managedForm.addPart(spart);
		tv = new CheckboxTreeViewer(tree);
		tv.setContentProvider(new RuleBaseContentProvider());
		tv.setLabelProvider(new RuleBaseLabelProvider());
		tv.setSorter(new ViewerSorter());
		//tv.addFilter(new RuleViewerFilter());
		tv.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				managedForm.fireSelectionChanged(spart, event.getSelection());
			}
		});

		tv.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				Object element = event.getElement();
				boolean checkState = event.getChecked();
				tv.setSubtreeChecked(element, checkState);
				if (element instanceof IRuleBase) {
					((IRuleBase) element).setEnabled(checkState);
					;
					IRuleBaseDescriptor[] descriptors = RuleBaseManager
							.getInstance().getRuleBaseDescriptors();
					for (int i = 0; i < descriptors.length; i++) {
						if (((DocumentRuleBaseDescriptor) descriptors[i])
								.getRuleBaseName().equals(
										((IRuleBase) element).getName())) {
							String[] targets = ((DocumentRuleBaseDescriptor) descriptors[i])
									.getTargets();
							for (int j = 0; j < targets.length; j++) {
								RuleBaseManager.getInstance()
										.removeLinksToTarget(targets[j]);
							}
						}
					}

					IRuleSet[] sets = ((IRuleBase) element).getRuleSets();
					for (int i = 0; i < sets.length; i++) {
						sets[i].setEnabled(checkState);
					}
				} else if (element instanceof IRule) {
					((IRule) element).setEnabled(checkState);
				}
			}
		});
		loadViewer(false);

	}

	@Override
	protected void createToolBarActions(IManagedForm managedForm) {
		ValidationUiPlugin plugin = ValidationUiPlugin.getDefault();
		plugin.registerImage(ValidationUIConstants.HORIZONTAL_ICON_ID,
				HORIZONTAL_ICON_FILENAME);
		plugin.registerImage(ValidationUIConstants.VERTICAL_ICON_ID,
				VERTICAL_ICON_FILENAME);
		plugin.registerImage(ValidationUIConstants.COLLAPSE_ALL_ICON_ID,
				COLLAPSE_ALL_ICON_FILENAME);
		plugin.registerImage(ValidationUIConstants.EXPAND_ALL_ICON_ID,
				EXPAND_ALL_ICON_FILENAME);
		final ScrolledForm form = managedForm.getForm();

		// horz/vertical form buttons
		Action horzAction = new Action("Horizontal layout",
				Action.AS_RADIO_BUTTON) {
			public void run() {
				sashForm.setOrientation(SWT.HORIZONTAL);
				form.reflow(true);
			}
		};
		horzAction.setChecked(true);
		horzAction.setToolTipText("Horizontal layout");
		horzAction.setImageDescriptor(plugin
				.getImageDescriptor(HORIZONTAL_ICON_FILENAME));
		Action vertAction = new Action("Vertical layout",
				Action.AS_RADIO_BUTTON) {
			public void run() {
				sashForm.setOrientation(SWT.VERTICAL);
				form.reflow(true);
			}
		};

		vertAction.setChecked(false);
		vertAction.setToolTipText("Vertical layout");
		vertAction.setImageDescriptor(plugin
				.getImageDescriptor(VERTICAL_ICON_FILENAME));
		form.getToolBarManager().add(horzAction);
		form.getToolBarManager().add(vertAction);

		// expand/collapse all buttons
		Action expandAllAction = new Action("Expand all",
				Action.AS_RADIO_BUTTON) {
			public void run() {
				tv.expandAll();
			}
		};
		expandAllAction.setChecked(false);
		expandAllAction.setToolTipText("Expand all nodes in the tree");
		expandAllAction.setImageDescriptor(plugin
				.getImageDescriptor(EXPAND_ALL_ICON_FILENAME));
		Action collapseAllAction = new Action("Collapse all",
				Action.AS_RADIO_BUTTON) {
			public void run() {
				tv.collapseAll();
			}
		};
		collapseAllAction.setChecked(true);
		collapseAllAction.setToolTipText("Collapse all nodes in the tree");
		collapseAllAction.setImageDescriptor(plugin
				.getImageDescriptor(COLLAPSE_ALL_ICON_FILENAME));
		form.getToolBarManager().add(expandAllAction);
		form.getToolBarManager().add(collapseAllAction);

	}

	@Override
	protected void registerPages(DetailsPart detailsPart) {
		detailsPart.setPageLimit(10);
		rulebaseDetailsPage = new RulebaseDetailsPage();
		detailsPart
				.registerPage(RulebaseDetailsPage.class, rulebaseDetailsPage);
		ruleDetailsPage = new RuleDetailsPage();
		detailsPart.registerPage(RuleDetailsPage.class, ruleDetailsPage);
		detailsPart.setPageProvider(new RuleViewerDetailsPageProvider());
	}

	private ArrayList<IRuleBase> getRuleBases(boolean forceReload) {
		rbMgr = RuleBaseManager.getInstance();
		ArrayList<IRuleBase> ruleBases = new ArrayList<IRuleBase>();
		for (int i = 0; i < rbMgr.getRuleBaseDescriptors().length; i++) {
			DocumentRuleBaseDescriptor rbDesc = (DocumentRuleBaseDescriptor) rbMgr
					.getRuleBaseDescriptors()[i];
			String rbName = rbDesc.getRuleBaseName();
			// temporarily eliminate the IA2 and MSAA rulebases from the viewer
			// remove this test after they have been cleaned up
			if (!(rbName.startsWith("MSAA") || rbName.startsWith("IA2"))) {
				if (forceReload){
					rbMgr.removeRuleBase(rbName);
				}
				IRuleBase rBase = rbMgr.getRuleBase(rbName);
				if (rBase != null) {
					ruleBases.add(rBase);
				} else {
					LoggingUtil.println(IReporter.ERROR,
							"Unable to load rulebase " + rbName);
				}
			}
		}
		return ruleBases;
	}

	public static CheckboxTreeViewer getCheckBoxTreeViewer() {
		return tv;
	}

	private void loadViewer(boolean forceReload) {
		if (tv != null) {
			tv.setInput(getRuleBases(forceReload));
			// We want to start out with all rulebases and rules enabled
			tv.setAllChecked(true);
			// the following is necessary because setAllChecked
			// does not fire a checkStateChangedEvent
			Object[] checked = tv.getCheckedElements();
			for (int i = 0; i < checked.length; i++) {
				tv.setSubtreeChecked(checked[i], true);
			}
		}
	}

}
