/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Mike Squillace - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.validation.valdoc.scripting;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.LinkedList;

import org.eclipse.actf.model.IModel;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.ContextFactory;

/**
 * A code interpreter for scripts written in JavaScript/Rhino.
 * 
 * @author Mike Squillace
 */
public class JavaScriptCodeInterpreter extends AbstractCodeInterpreter {

	/**
	 * create a new interpreter for JavaScript
	 * 
	 * @param lang
	 * @param model
	 */
	public JavaScriptCodeInterpreter(String lang, IModel model) {
		super(lang, model);

		ContextFactory cf = ContextFactory.getGlobal();

		//javascript processor can only be initialized once  
		if (cf.getApplicationClassLoader() == null) {
			String[] paths = classpath.split(File.pathSeparator);
			LinkedList urlList = new LinkedList();
			for (String path : paths) {
				try {
					URL url = new File(path).toURL();
					urlList.add(url);
				} catch (MalformedURLException e) {
				}
			}

			cf.initApplicationClassLoader(new URLClassLoader((URL[]) urlList
					.toArray(new URL[urlList.size()]), getClass()
					.getClassLoader()));
		}
		if (Context.getCurrentContext() == null) {
			Context.enter();
		}
	}

	protected void initClasspath() {
	}

	protected String formatPackageImport(String packageName) {
		return "importPackage(Packages." + packageName + ")\n";
	}

	/** {@inheritDoc} */
	protected Object toJavaObject(Object o, Class c) {
		return o;
	}

	/** {@inheritDoc} */
	protected Object[] toJavaObjects(Object o, Class[] paramTypes) {
		return new Object[] { o };
	}

	protected void finalize() throws Throwable {
		try {
			Context.exit();
		} finally {
			super.finalize();
		}
	}

} // JavaScriptCodeInterpreter
