/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.validators;

import org.eclipse.actf.validation.rules.IRuleSet;

/**
 * implemented by users who wish to validate a component, and to verify that
 * certain features and properties of these components are in accord with
 * pre-defined constraints as specified in a <code>IRuleSet</code> within a
 * <code>IRuleBase</code>, such as an XML validation document. An example is
 * packaged with this application for performing validation of Swing GUIs with
 * respect to their degree of accessibility.
 * 
 * @see org.eclipse.actf.validation.rulebase.IRuleBase
 * @see org.eclipse.actf.validation.rules.IRuleSet
 * @author John Buslawski
 */
public interface IValidator {

	/**
	 * set the rule set to be used for validating the components
	 * 
	 * @param ruleSet
	 */
	public void setRuleSet(IRuleSet ruleSet);

	/**
	 * return the ruleSet being used by this validator
	 * 
	 * @return ruleSet instance
	 */
	public IRuleSet getRuleSet();

	/**
	 * validates an individual component. This method is called by the
	 * <code>IValidationProcess</code> in response to
	 * <code>ExecutionPoint</code> objects.
	 * 
	 * @param component -- component to validate
	 * @see org.eclipse.actf.validation.core.ValidationProcessManager
	 */
	public void validate(IValidationContext context, Object component);
	
} // IValidator
